import { ActionFormData } from "@minecraft/server-ui";
import { system, world } from '@minecraft/server';
import { getPlayerTagData, getTagValue } from '../systemFiles/tagUtils';

const ENTITY_TYPE = "minecraft:armor_stand";
const ENTITY_NAME = "list_players";
const PLAYER_TAG_PREFIX = "playerData:";

system.afterEvents.scriptEventReceive.subscribe((event) => {
    if (event.id === "fearcube:forms") {
        if (event.message === "list_players") {
            updatePlayerLists();
            showPlayerListForm(event.sourceEntity);
        } else if (event.message.startsWith("profile:")) {
            const playerName = event.message.split(":")[1];
            const targetPlayer = Array.from(world.getPlayers()).find(p => p.name === playerName);
            if (targetPlayer) {
                showPlayerProfileForm(targetPlayer, event.sourceEntity);
            } else {
                const playerData = getPlayerData(playerName);
                if (playerData) {
                    showOfflinePlayerProfileForm(playerData, event.sourceEntity);
                }
            }
        }
    }
});

function getEntity() {
    const entities = world.getDimension("overworld").getEntities();
    return entities.find(e => e.typeId === ENTITY_TYPE && e.nameTag === ENTITY_NAME);
}

function getPlayerData(playerName) {
    const entity = getEntity();
    if (!entity) return null;

    const tags = entity.getTags();
    const playerTag = tags.find(tag => tag.startsWith(PLAYER_TAG_PREFIX) && tag.includes(`"name":"${playerName}"`));
    return playerTag ? JSON.parse(playerTag.substring(PLAYER_TAG_PREFIX.length)) : null;
}

function updatePlayerLists() {
    const entity = getEntity();
    if (!entity) return;

    const currentPlayers = Array.from(world.getPlayers());
    const currentOnlinePlayerNames = currentPlayers.map(player => player.name);

    const tags = entity.getTags();
    const allPlayerTags = tags.filter(tag => tag.startsWith(PLAYER_TAG_PREFIX));

    let playersData = {};

    allPlayerTags.forEach(tag => {
        const playerData = JSON.parse(tag.substring(PLAYER_TAG_PREFIX.length));
        playersData[playerData.name] = playerData;
    });

    currentOnlinePlayerNames.forEach(name => {
        const player = currentPlayers.find(p => p.name === name);
        if (!playersData[name]) {
            playersData[name] = { name: name, online: true };
        } else {
            playersData[name].online = true;
        }

        playersData[name].lvl = getTagValue(player, "lvl") ?? 0;
        playersData[name].xp = getTagValue(player, "experience") ?? 0;
        playersData[name].nextLvlXp = getTagValue(player, "next_exp") ?? 0;
        playersData[name].money = getTagValue(player, "money") ?? 0;
        playersData[name].playGame = getTagValue(player, "play_game") ?? 0;
        playersData[name].playTimeSeconds = getTagValue(player, "second") ?? 0;
        playersData[name].playTimeHours = Math.floor(playersData[name].playTimeSeconds / 3600);
        playersData[name].playTimeMinutes = Math.floor((playersData[name].playTimeSeconds % 3600) / 60);
    });

    for (let name in playersData) {
        if (!currentOnlinePlayerNames.includes(name)) {
            playersData[name].online = false;
        }
    }

    allPlayerTags.forEach(tag => entity.removeTag(tag));
    for (let name in playersData) {
        entity.addTag(`${PLAYER_TAG_PREFIX}${JSON.stringify(playersData[name])}`);
    }
}

function showPlayerListForm(player) {
    const entity = getEntity();
    if (!entity) return;

    const tags = entity.getTags();
    const allPlayerTags = tags.filter(tag => tag.startsWith(PLAYER_TAG_PREFIX));

    let playersData = allPlayerTags.map(tag => JSON.parse(tag.substring(PLAYER_TAG_PREFIX.length)));

    if (playersData.length === 0) return;

    const currentPlayerName = player.name;

    playersData.sort((a, b) => {
        if (a.name === currentPlayerName) return -1;
        if (b.name === currentPlayerName) return 1;
        return a.online === b.online ? 0 : a.online ? -1 : 1;
    });

    let form = new ActionFormData()
        .title("%fearcube.profile_listplayers")
        .body("%fearcube.profile_listplayersinfo");

    let buttonNames = [];

    playersData.forEach(playerData => {
        let displayName;
        const texture = playerData.online ? "textures/icon/other/player_online" : "textures/icon/other/player_offline";
        if (playerData.name === currentPlayerName) {
            displayName = `${playerData.name}\n%fearcube.you`;
        } else {
            const status = playerData.online ? "%fearcube.profile_player" : "§cOffline";
            displayName = `${playerData.name}\n${status}`;
        }
        form.button(displayName, texture);
        buttonNames.push(playerData.name);
    });

    form.show(player).then(response => {
        const selection = response.selection;
        if (selection !== undefined) {
            const selectedPlayerName = buttonNames[selection];
            const targetPlayer = Array.from(world.getPlayers()).find(p => p.name === selectedPlayerName);
            if (targetPlayer) {
                showPlayerProfileForm(targetPlayer, player);
            } else {
                const playerData = getPlayerData(selectedPlayerName);
                if (playerData) {
                    showOfflinePlayerProfileForm(playerData, player);
                }
            }
        }
    }).catch((e) => {
        console.error(e.stack);
    });
}

function showOfflinePlayerProfileForm(offlinePlayer, requestingPlayer) {
    const playTimeSeconds = offlinePlayer.playTimeSeconds ?? 0; // Проверка на undefined
    const playTimeHours = offlinePlayer.playTimeHours ?? 0;     // Проверка на undefined
    const playTimeMinutes = offlinePlayer.playTimeMinutes ?? 0; // Проверка на undefined

    let profile_form = new ActionFormData();
    profile_form.title(`%fearcube.profile_player ${offlinePlayer.name}`);
    profile_form.body(`\n              \n\n\n\n§7%fearcube.nickname: §a${offlinePlayer.name} \n§7%fearcube.lvl: §a${offlinePlayer.lvl}§7 | §6${offlinePlayer.xp}§7/§c${offlinePlayer.nextLvlXp}§7xp\n§7%fearcube.play_gms: §c${offlinePlayer.playGame}\n§7%fearcube.balance: §c${offlinePlayer.money}\n§7%fearcube.time_online: §a${playTimeHours} §7%fearcube.time_online_hours §a${playTimeMinutes} §7%fearcube.time_online_minutes\n§7`);
    profile_form.button("%dr.button.ok ");

    profile_form.show(requestingPlayer).then(r => {}).catch((e) => { 
        console.error(e.stack); 
    });
}

function showPlayerProfileForm(targetPlayer, requestingPlayer) {
    const objective_lvl_score = getTagValue(targetPlayer, "lvl") ?? 0;
    const objective_xp_score = getTagValue(targetPlayer, "experience") ?? 0;
    const objective_xp_up_score = getTagValue(targetPlayer, "next_exp") ?? 0;
    const objective_money_score = getTagValue(targetPlayer, "money") ?? 0;
    const objective_game_player_score = getTagValue(targetPlayer, "play_game") ?? 0;
    
    // Проверяем, что данные времени не undefined
    const objective_time_seconds_player_score = getTagValue(targetPlayer, "second") ?? 0;
    const objective_time_hours_player_score = Math.floor(objective_time_seconds_player_score / 3600) ?? 0;
    const objective_time_minutes_player_score = Math.floor((objective_time_seconds_player_score % 3600) / 60) ?? 0;

    let profile_form = new ActionFormData();
    profile_form.title(`%fearcube.profile_player ${targetPlayer.name}`);
    profile_form.body(`\n              \n\n\n\n§7%fearcube.nickname: §a${targetPlayer.name} \n§7%fearcube.lvl: §a${objective_lvl_score}§7 | §6${objective_xp_score}§7/§c${objective_xp_up_score}§7xp\n§7%fearcube.play_gms: §c${objective_game_player_score}\n§7%fearcube.balance: §c${objective_money_score}\n§7%fearcube.time_online: §a${objective_time_hours_player_score} §7%fearcube.time_online_hours §a${objective_time_minutes_player_score} §7%fearcube.time_online_minutes\n§7`);
    profile_form.button("%dr.button.ok ");

    profile_form.show(requestingPlayer).then(r => {}).catch((e) => {
        console.error(e.stack);
    });
}

